#include "opencv2/opencv.hpp"

void drawHistogram(const cv::Mat& input, cv::Mat& histdisp, int p1, int p2)
{
    int histwork[256];
    for (int i = 0; i < 255; i++) {
        histwork[i] = 0;
    }

    for (int j = 0; j < input.rows; j++) {
        for (int i = 0; i < input.cols; i++) {
            histwork[input.at<uchar>(j, i)]++;
        }
    }

    cv::cvtColor(input, histdisp, CV_GRAY2BGR);
    for (int i = 0; i < 255; i++) {
        cv::line(histdisp, 
                 cv::Point(i, input.rows - 1),
                 cv::Point(i, input.cols - histwork[i] / 8),
                 CV_RGB(0, 255, 0));
    }

    cv::rectangle(histdisp,
                  cv::Point(p1, input.rows - 3),
                  cv::Point(p2, input.rows - 1),
                  CV_RGB(255, 0, 0), CV_FILLED);
}

uchar pix_value_conversion(uchar val, int p1, int p2)
{
    if (val < p1) {
        return 0;
    } else if (p1 <= val && val <= p2) {
        return (255 * (val - p1)) / (p2 - p1);
    } else {
        return 255;
    }
}

int main()
{
    cv::namedWindow("input", cv::WINDOW_AUTOSIZE);
    cv::namedWindow("output", cv::WINDOW_AUTOSIZE);
    cv::VideoCapture cap(0);
    cv::Mat input;
    cap >> input;
    cv::Mat output(input.rows, input.cols, CV_8U);
    cv::Mat histdisp(input.rows, input.cols, CV_8UC3);

    int p1 = 0, p2 = 255;
    cv::createTrackbar("p1", "input", &p1, 255);
    cv::createTrackbar("p2", "input", &p2, 255);

    while (1) {
        cap >> input;
        cv::cvtColor(input, input, CV_BGR2GRAY);

        if (p2 <= p1) {
            p2 = p1 + 1;
        }

        drawHistogram(input, histdisp, p1, p2);

        for (int j = 0; j < input.rows; j++) {
            for (int i = 0; i < input.cols; i++) {
                output.at<uchar>(j, i) 
                    = pix_value_conversion(input.at<uchar>(j, i), p1, p2);
            }
        }

        cv::imshow("input", histdisp);
        cv::imshow("output", output);
        if (cv::waitKey(30) > 0) {
            break;
        }
    }

    return 0;
}
